/*
 * Dual macros to perform interrupts (no extra CALL)
 *
 * Example:
 *
 *		INTERRUPT(myint) func()
 *		{
 *			int local_variable;
 *			... code here ...
 *			INTRET
 *		}
 *
 * You must initialize the appropriate 8086 interrupt vector with
 * your code segment (CS) and the offset to the symbol 'myint'.
 *
 * When using these MACRO's, the function must return with INTRET.
 * Place INTRET at the end of your function, and also use it in
 * place of any 'return' statements. DO NOT use 'return'!
 *
 * 'func()' may NOT be called from the mainline program.
 *
 * This file **REQUIRES** the extended pre-processor (MCP).
 * (CC86 ... -P, or "Preprocess" step enabled in DDSIDE)
 */
#define	_SP_
#define	INTERRUPT(name) asm {\
name	PUSH	AX\
		PUSH	BX\
		PUSH	CX\
		PUSH	DX\
		PUSH	SI\
		PUSH	DI\
		PUSH	ES\
		PUSH	DS\
		PUSH	CS\
		POP		DS\
		MOV		AX,?temp\
		PUSH	AX\
}
#define INTRET asm {\
		MOV		SP,BP\
		POP		BP\
		POP		AX\
		MOV		_SP_?temp,AX\
		POP		DS\
		POP		ES\
		POP		DI\
		POP		SI\
		POP		DX\
		POP		CX\
		POP		BX\
		POP		AX\
		IRET\
}
#undef _SP_
