*
* Allocate a block of memory: char *malloc(size)
*
malloc	PUSH	BP		Save callers stack frame
	MOV	BP,SP		Address parameters
* Search for a free block of memory
	MOV	SI,#?heap	Begin here
?1	MOV	AL,[SI]		Get flag
	AND	AL,AL		Free memory?
	JZ	?4		Yes, allocate it here
	MOV	BX,1[SI]	Get block size
	DEC	AL		Un-allocated?
	JNE	?2		No, its busy
* Found free block, see if its large enough
	CMP	BX,4[BP]	Is it large enough?
	JAE	?3		Yes, its ok
* This block is not suitable, advance to next
?2	LEA	SI,3[BX+SI]	Skip to next block
	JMP	<?1		and continue
* This block is ok to re-allocate
?3	SUB	BX,4[BP]	Calculate remaining
	SUB	BX,#3		Convert for overhead
	JBE	?6		Too small, Don't split up
* Split this block into two blocks
	MOV	CX,BX		Save for later
	MOV	BX,4[BP]	Get requested size
	MOV	<3[BX+SI],#1	Indicate free
	MOV	4[BX+SI],CX	Set new size
	JMP	<?5		and exit
* Allocate on the end of memory
* Check that enough room is available
?4	MOV	BX,4[BP]	Get requested size
	MOV	CX,SP		Get current stack pointer
	SUB	CX,#1000	Adjust for protective margin
	SUB	CX,BX		Adjust for buffer size
	XOR	AX,AX		Assume it will fail
	CMP	CX,SI		Is there enough room?
	JBE	?7		No, return zero
* Ok to proceed and perform memory allocation
	MOV	<3[BX+SI],#0	Indicate end of list
?5	MOV	1[SI],BX	Set size
?6	MOV	<[SI],#2	Flag as allocated
	LEA	AX,3[SI]	Get address
?7	POP	BP		Restore caller
	RET
*
* Release a block of memory: free(ptr)
*
free	MOV	BX,SP		Get stack pointer
	MOV	CX,2[BX]	Get parameter
	SUB	CX,#3		Backup to "real" beginning
* Search the allocation list for this block
	MOV	SI,#?heap	Point to start of heap
?8	MOV	AL,[SI]		Get link
	AND	AL,AL		End of list?
	JZ	?10		Yes, clean up and exit
	CMP	SI,CX		Is this it?
	JZ	?9		Yes, we have it
	MOV	BX,1[SI]	Get size
	LEA	SI,3[BX+SI]	Advance to next
	JMP	<?8		And try again
* Mark the block as un-allocated
?9	MOV	<[SI],#1	Release memory
*
* Garbage collection, scan the memory allocation blocks, and
* convert any contiguous un-allocated blocks to single areas.
* Also, truncate the list at the last allocated block.
*
?10	MOV	SI,#?heap	Point to memory area
* Check this block for allocation state
?11	MOV	AL,[SI]		Get allocation flag
	AND	AL,AL		End of list?
	JZ	?16		Yes, quit
	MOV	BX,1[SI]	Get size of block
	DEC	AL		Free?
	JNZ	?14		No, its not
* This block is free, check following
?12	MOV	AL,3[BX+SI]	Get next block flag
	AND	AL,AL		End of list
	JZ	?15		Yes, truncate
	DEC	AL		Allocated?
	JNZ	?13		Yes, stop looking
* Next block is also free
	ADD	BX,4[BX+SI] Add next block size
	ADD	BX,#3		Add next block overhead
	JMP	<?12		And keep looking
* Resave this block size
?13	MOV	1[SI],BX	Save new size
* Advance to next block
?14	LEA	SI,3[BX+SI]	Advance to next
	JMP	<?11		And continue
* Mark this block as the end of the list
?15	MOV	<[SI],#0	Indicate end of list
?16	RET
